; Lader.asm V2.5.1 Multi-IO-Karte und Software FIFO
;
; Intel-Hex Datei lesen, zeilenweise interpretieren
; (Zieladresse wird dem Hex-File entnommen)
;
; Stand:
; 15.01.2018	V2.5.1 Buffer/FIFO auf Systemspeicher (2D00h) gelegt
; 12.01.2018	V2.5 Test OK
; 09.01.2018	V24 mit Software-FIFO und Multi-IO
; 26.06.2016 	V24 Anpassungen # V3.5 fuer CF-Test-V3.5
; 21.04.2016	Hex-Vorpann testen, nach 16Byte Fehlermeldung
; 19.04.2016    Umstellung auf Seriellesinterface der CF-Karte
;
; 14.03.2015	Mit "Bootlader" aus EPROM 4000h ins RAM ab 2900h kopieren  
; 12.03.2015	Leseroutine auf Zeilenbuffer umgestellt
; 17.02.2015	UART-Init ueberarbeitet und direkt am Programmanfang gesetzt
; 01.02.2015	auf 04000h assembliert - alter Platz vom Basicinterpreter
; 30.01.2015	Hex-Interpreter integriert
;		  keine Ueberpruefung der Buffergrenzen
; 27.01.2015	Zeitfenster eingebaut
;		  (Empfang wartet max 3sek auf Daten)
;		  (Senden wartet max 12ms auf Empfaenger)
;		  Buffergrenze auf 16kByte gesetzt
;		  Meldetexte
;
;-----------------------------------------------------------------------------
;
; PIO1 Centronics und UNI-IO Kanal
;
pio1a	.equ	03h	; Centronic's
pio1b	.equ	13h	; frei
pio1c	.equ	23h	; Steuerung Centronics
pio1s	.equ	33h	; Steuerregister
bap1	.equ	0abh	; 1010 1011
			; Kanal A Modus 1 Ausgabe, Kanal B Modus 0 Eingabe
			; Kanal C Eingabe
;
; PIO2 IO-Byte und Systemuhr
;
pio2a	.equ	43h	; 
pio2b	.equ	53h	; IO-byte DIP-Schalter			<--		
pio2c	.equ	63h	; RTC 58321 Control
pio2s	.equ	73h	; Steuerregister
bap2	.equ	82h	; 1000 0010
bap2rdu	.equ	92h	; 1001 0010
;
;--------------------------------------------------------------------
; Baudraten
; Wert als BCD, da Timer auf BCD Zaehler eingestellt wird
;
; fuer 6,144Mhz
;
; 9600Bd = 20h lowByte
; 4800Bd = 40h .
; 2400Bd = 80h .
; 1200Bd = 60h lowByte
;	   01h highByte
;--------------------------------------------------------------------

tim0	.equ	83h	; Timer Reg fuer com1 BD-Rate
tim1	.equ	93h	; Timer Reg fuer com2 BD-Rate
tim2	.equ	0a3h	; frei
tims	.equ	0b3h	; Steuerregister
			; Timer0-2 Betriebsarten
			; als BCD-Zaehler, Rechteckgenerator,
			; erst Timewert-Low, dann Timewert-High laden
bati0	.equ	37h	; 0011 0111 
bati1	.equ	77h	; 0111 0111
bati2	.equ	0b7h	; 1011 0111
;
com1s	.equ	0c3h	; Steuerregister 
com1d	.equ	0d3h	; Daten com1
bacom1	.equ	15h	; Betriebsart (8bit,1Stop,ohne Paritaet, Takt/16)		;
;
com2s	.equ	0e3h	; Steuerregister
com2d	.equ	0f3h	; Daten com2
bacom2	.equ	15h	; Betriebsart (8bit,1Stop,ohne Paritaet, Takt/16)
;
;
;-----------------------------------------------------------------------------
;
SRCBEG	.equ 2FBFh	; Startadresse
SRCEND	.equ 2FC1h	; Endadresse
DSTBEG	.equ 2FC3h	; Anfang Zieladresse/Parameter
DSTEND	.equ 2FC5h	; 
RAMBEG	.equ 37B0h	;
;
; Monitorprogramme
;
vwarm	.equ 1003h	; Videomonitor
co	.equ 0049h	; (C) Monitorausgabe
ci	.equ 0043h	; (A) Tastencode, Rueksprung erst nach Betaetigung, wartet nicht auf loslassen
csts	.equ 0052h	; Status Tastatur, gedruekt Z=0, sonst Z=1
dely1	.equ 000bh	; warte 1ms
copy	.equ 0088h	; SRCBEG=Quelle start,SRCEND=Quelle End,DSTBEG=Zielstart
onesec	.equ 001bh	; warte 1 Sekunde
string	.equ 1021h	; Textausgabe bis 00h
adrot	.equ 103Ch	; (DE) vierstellig ausgeben (HL)=Position
error	.equ 1027h       ; Error Videomonitor
;
	.org 2f8ch	; Sprungtabelle für IRQ- Serviceroutinen

intrap:	.ds	3h	; Software IRQ
int3:	.ds	3h	;
int4:	.ds	3h	;
int5:	.ds	3h	;
int55:	.ds	3h	; Hardware IRQ
int6:	.ds	3h	;
int65:	.ds	3h	; Hardware IRQ
int7:	.ds	3h	;
int75:	.ds	3h	; Hardware IRQ
;
;==================================================================
;
fifo	.equ 2D10h	; Softwarefifo
buffer	.equ 2D50h	; Hex-Zeilenbuffer
baud	.equ 0020h	; fuer 9600Bd
timeout .equ 0800h	; ca 2s
;
fiforts	.equ 28h 	; Bufferposition fuer RTS
fifomax	.equ 40h		; Fifo Groesse
;
; Zwischenspeicher fuer Buffer(FIFO) Steuerung
;
	.org 2D00h
buffz	.dw  0000h	; Schreibzeiger fuer Zielbuffer
wrz	.dw  0000h	; Schreibzeiger fuer IRQ
rdz	.dw  0000h	; Lesezeiger	
v24flag	.db  00h		; 1000 0001
;			; :  : :  :
;			; :  : :  +--- RTS punkt erreicht
;			; :  : +------ Timeout   
;			; :  +-------- INIT Flag 0=FIFO Zeiger setzen
;			; :                      1=FIFO Betrieb 
;			; +---------- Buffer voll
;			;
;
	.org 8000h	; 

;	
lader:	jmp lade	; Intel-Hex empfangen, interpretieren
Ende:	jmp vwarm	; zurueck zum Monitor
;
;====================================================================
; Hauptprogramm
;
; - laden Intel-Hex-File Zeilenweise
; - Intel-Hex Interpreter
;
lade:	call	mioini		; Karte initialisieren
	lxi  	h,meld_0	;
	call 	string		;
	lxi	h,buffer	; Zeiger einrichten
	shld	buffz		;
	xra	a		;
	sta	v24flag		;
	call 	onesec		; Wartezeit bis Datenabgeholt werden
	call	onesec		;
	call	onesec		;
	;
	call	irqi		; IRQ Serviceroutinen einrichten
	mvi	a,95h		; 1001 0101 = RX IRQ und
	out	com1s		; RTS freigeben
	;
lade_l:	lxi  	h,buffer	;
	call 	read		; Zeile empfangen
	jnz  	lade_e4		; Bufferueberlauf
	lda  	v24flag		; Timeout ?
	ani  	08h		;
	jnz  	lade_e5		;
	;
	lxi  	h,buffer	; Buffer auf Anfang
	call 	hxi		; eine Zeile interpretieren und im Zielbuffer ablegen
	;
	mvi 	c,2ah		; Sternchen pro Zeile ausgeben
	call 	co		;
	;
	cpi 	01h		; Fehlercode auswerten
	jz 	lade_e1		; kein ASCII
	cpi 	02h		;
	jz 	lade_e2		; keine Startmarke
	cpi 	03h		;
	jz 	lade_e3		; falsche Pruefsumme
	cpi 	04h
	jnz 	lade_l		; weiter bis Dateiende
	;
	lxi 	h, meld_5	; alles OK zurueck zum Monitor
lade_e: call 	string		; Bildschirmmeldung ausgeben
	jmp 	Ende		;
	
lade_e1: lxi 	h,err_1		; kein ASCII
	jmp 	lade_e		;
	
lade_e2: lxi 	h,err_2		; keine Startmarke
	jmp 	lade_e		;
		
lade_e3: lxi 	h,err_3		; falsche Pruefsumme
	jmp 	lade_e

lade_e4: lxi 	h,err_0		; Uebertragungsfehler
	jmp  	lade_e
	
lade_e5: lxi 	h,err_5		; Timeout
	jmp  	lade_e
	
;====================================================================
;
; Zeile empfangen (bis LF max 64 Zeichen)
;

read:	lxi  h,buffer		; Buffer
	mvi  b,40h		; maxbuffer
	
str_l:	call rx			; Byte holen
	mov  m,a		;      speichern
	cpi  0ah		; ? LF
	jz   str_end		; Zeilenende
	inx  h			;
	dcr  b			;
	jnz  str_l		; weiter
str_er:	mvi  a,01h		; Fehler Bufferueberlauf
	ret			; Fehlercode 01h

str_end: xra a			; Zeile vollständig empfangen
	ret			; Fehlercode 00h
;		
;--------------------------------------------------------------------
;
; Rx 	Daten aus FIFO lesen
;
; (A)=Zeichen
; (A)=0 und Z=1 Timeout V24-Flag 0000 1000b
;
rx:	push	b		;
	push	d		;
	push	h		;
	lda	v24flag		;
	ani	10h		; Init-Flag ?
	cz	setirq		; nein -> Zeiger auf Anfang
rxz_l1	lxi	b,timeout	; Abbruchbedingung laden
rxz_l2:	di			;
	lhld	wrz		; Daten in FIFO ?
	xchg			;
	lhld	rdz		;
	mov	a,l		;
	cmp	e		;
	jc	rxzrb		; ja --> Daten lesen
	lda	v24flag		;
	ani	01h		; RTS gesetzt ?
	jnz	rtscl		; RTS und IRQ freigeben
	ei			;
	lda	v24flag		;
	ani	080h		; FIFO Ende ?
	jnz	rxz_e		; ja --> Ende mit Bufferueberlauf
	call	dely1		;
	dcr	c		; Timeout ?
	jnz	rxz_l2		;
	dcr	b		;	
	jnz  	rxz_l2		; warten
	lda	v24flag		;
	ori	08h		; 0000 1000 Timeout
	sta	v24flag		;
	xra	a		; keine Daten im FIFO (A)=0 Z=1
	
rxz_e:	pop	h		;	
	pop	d		;
	pop	b		;
	ret			; Ende RX
	
;
;
; setirq	IRQ, FIFO, Flags setzen
;
setirq:	lxi	h,fifo		;  
	shld	wrz		;
	shld	rdz		; auf Anfang
	mvi	a,10h		; 
	sta	v24flag		; Init-Flag setzen
	ret			;
;	
; Buffer auslesen
;
rxzrb:	mov	a,m		; Byte holen
	inx	h		;
	shld	rdz		;
	ei			;
	jmp	rxz_e		; warten auf naechstes Byte

;
; RTS und IRQ freigeben
;
rtscl:	lxi	h,fifo		; Buffer 
	shld	wrz		;
	shld	rdz		; auf Anfang
	lda	v24flag		;
	ani	0feh		; 1111 1110b
	sta	v24flag		; RTS-Flag loeschen
	mvi	a,95h		; 1001 0101 = RX IRQ und
	out	com1s		; RTS freigeben
	ei
	jmp	rxz_l1		;
;
;
;--------------------------------------------------------------------
;
; IRQ vorbereiten
;
irqi:	lxi	h,int75		; Spungtabelle
	mvi	m,0c3h		; = JMP
	inx	h		; IRQ Seviceroutine
	lxi	d,irqs		; In Sprungtabelle
	mov	m,e		;
	inx	h		;
	mov	m,d		; eintragen
	mvi	a,1bh		; 0001 1011 irq7.5 freigeben
	sim			;
	ei			; IRQ freigeben
	ret			;
;
; IRQ Serviceroutine
; Ein Byte uber V24 empfangen und 
; im Empfangsbuffer (FIFO) ablegen
;
irqs: 	push	psw		; alles sichern
	push	h		;
	push	d		;
	push	b		;
	in	com1s		; irq loeschen
	ani 	01h		; Empfangsregister voll ?
	jz	irqe		; nein
	lhld	wrz		; Zeiger auf Zeilenbuffer	
	in	com1d		;
	mov	m,a		; daten ablegen
	inx	h		;
	shld	wrz		; Zeiger sichern
	mvi	a,fiforts	; Zeiger an RTS Grenze
	cmp	l		;
	jz	setrts		; RTS Setzen
	mvi	a,fifomax	; Test auf Ende Zeilenbuffer
	cmp	l		;
	jz	irqbs		; Zeilenbuffer Ende erreicht
	
irqe:	pop	b		; alles wieder zurueck
	pop	d		;
	pop	h		;
	pop	psw		;
	ei			; IRQ freigeben
	ret			;
	
setrts:	mvi	a,0d5h		; RTS gesetzt
	out	com1s		; IRQ Freigabe
	lda	v24flag		;
	ori	01h		;
	sta	v24flag		; RTS-Flag setzen
	jmp	irqe		;

irqbs:	lxi	h,fifo		; Buffer wieder auf Anfang 
	shld	wrz		;
	mvi	a,55h		; RTS gesetzt
	out	com1s		; IRQ gesperrt
	lda	v24flag		;
	ori	81h		; 1000 0001 Error u. V24 Flag setzen
	sta	v24flag		;
	jmp	irqe		;	
;
;==================================================================
;
; Multi-IO Karte initialisieren
;		
mioini:	mvi	a,bap1		; PIO-Betriebsarten einstellen
	out	pio1s		;
	mvi	a,bap2		;
	out	pio2s		;
	xra	a		;
	out	pio2b		;
	out	pio2c		;
	;
	mvi	a,03h		; Reset com1 1
	out	com1s		;	
	mvi	a,bacom1	; com1-Betriebsart
	out	com1s		;
	mvi	a,55h		; RST On
	out	com1s		;
	;
	mvi	a,03h		; Reset com2 2
	out	com2s		;	
	mvi	a,bacom2	; com2-Betriebsart
	out	com2s		;
	mvi	a,55h		; RST On
	out	com2s		;	
	;
	mvi	a,bati0		; Timer-Betriebsarten
	out	tims		;
	mvi	a,bati1		;
	out	tims		;
	mvi	a,bati2		;
	out	tims		;
	;
	lxi	d,baud		; siehe oben
	mov	a,e		; in Timer 0 laden
	out	tim0		;
	mov	a,d		;
	out	tim0		;
	;
	mov	a,e		; in Timer 1 laden
	out	tim1		;
	mov	a,d		;
	out	tim1		;
	;
	mov	a,e		; in Timer 2 laden
	out	tim2		;
	mov	a,d		;
	out	tim2		;
	;
	ret	
;
;
;==============================================================================
; Eine Zeile Intel-Hex Daten aus RS232 Buffer lesen
;
; interpretieren und
; in Buffer legen
;
; eine Intel-Hex Zeile in 8Bit Systemen 
; ! jedes Byte ist in zwei ASCII-Zeichen codiert (z.B.10 = 31 30) !
;
;   I   Daten fuer Pruefsumme addieren inkl Pruefsumme = 0         I
;   I                                                           I
; : 10 2010 00 66 20 C3 88 23 C3 3B 24 D3 4B C9 1A D3 4B 13 C9 AF 0D 0A
; I I  I    I  I                                               I  I  I
; I I  I    I  1.Byte                                          I  I  Line-Feed
; I I  I    I                                                  I  I
; I I  I    Typ 00=Nutzdaten                                   I    CR
; I I  I        01=Ende der Inte-Hex datei (EOF)               I
; I I  I                                                       Pruefsumme
; I I  Startadresse
; I I
; I Anzahl Bytes
; I
; Startmarke
;
;
; Registerzuweisung:
;	
; (HL) Zeiger auf Lesebuffer
; (DE) Zeiger auf Schreibbuffer
; (C)  Bytezaehler
; (B)  Pruefsumme
;
;
;----------------------------------------------------
; Hex Interpreter wandelt eine Zeile Intel-HEX in Binaer-Code
; Anmerkung: Adressen werden pro Zeile ausgewertet, Zwischenraueme
;            werden nicht aufgefuellt - die Daten bleiben als Muell stehen 
; 
; Registerzuweisung:
;	
; (HL) Zeiger auf Lesebuffer
; (DE) Zeiger auf Schreibbuffer
; (C)  Bytezaehler
; (B)  Pruefsumme  
;
; Fehlercode:
;
; (A)=01h CY=1 kein ASII-Zeichen
; (A)=02h Z=1 Starmarke nicht gefunden
; (A)=02h Z=1 Pruefsumme falsch
; (A)=02h Z=1 Dateiende (EOF)
;
; Unterprogram: asc_h
;
; Startmarke ":" holen
;
hxi:	xra a		; Registervorbereiten (DE+BC)
	mov b,a		;
	mov c,a		;
	mov d,a		;
	mov e,a		;
	push b		;
	mvi c,0fh	; Fehlerzaehler setzen
hxi_sl:	mov a,m		; Startmarke suchen
	cpi 3ah		;
	jz hxi_ln	; weiter mit Laenge holen
			; Zeilenende(abspann) behandeln
	cpi 0Dh		; test auf CR
	jz hxi_mt	;
	dcr c		; Fehlerzaehler testen
	jz hxi_erm 	; keine Startmarke gefunden
hxi_mt:	inx h		; nachstes Byte holen
	jmp hxi_sl	;

; Laenge holen

hxi_ln:	pop b		;
	inx h		; 2.Byte holen
	call asc_h	; ASCII->HEX
	jc hxi_era	; Fehler kein ASCII
	mov c,a		; Bytezaehler laden
	add b		;
	mov b,a		; Pruefsumme schreiben
	
; Adresse holen

	inx h		; HighByte
	call asc_h
	jc hxi_era	; Fehler kein ASCII
	mov d,a		;
	add b		;
	mov b,a		; Pruefsumme schreiben
	
	inx h		; LowByte
	call asc_h
	jc hxi_era	; Fehler kein ASCII
	mov e,a		; 
	add b		;
	mov b,a		; Pruefsumme schreiben

; Type holen

	inx h
	call asc_h
	jc hxi_era	; Fehler kein ASCII
	cpi 00h		; Nutzdaten folgen
	jnz hxi_erf	; EOF erreicht
	add b		;
	mov b,a		; Pruefsumme schreiben
	
; Daten holen

hxi_dl: inx h		;
	mov a,m		;
	call asc_h	;
	jc hxi_era	; Fehler kein ASCII
	push psw
	add b		;
	mov b,a		; Pruefsumme schreiben
	pop psw
	xchg		;
	mov m,a		; Datenbyte in Zielbuffer
	xchg		;
	inx d		;
	dcr c		;
	jnz hxi_dl	; naechstes Byte

; Pruefsumme holen

	inx h		;
	mov a,m		;
	call asc_h	;
	jc hxi_era	; Fehler kein ASCII
	add b		;
	cpi 0h		;
	jnz hxi_ers	; Fehler Pruefsumme
	inx h		;
hxi_end: ret 			
;
; Fehlerbehandlung
;
hxi_era: ret		; (A)=01h CY=1 kein ASII-Zeichen

hxi_erm: xra a		; (A)=02h Z=1 Starmarke nicht gefunden
	mvi a,02h
	ret		;
	
hxi_ers: xra a		; (A)=03h Z=1 Pruefsumme falsch
	mvi a,03h
	ret		;
	
hxi_erf: xra a		; (A)=04h Z=1Dateiende (EOF)
	mvi a,04h
	ret
;
;----------------------------------------------------
;
; ASCII -> HEX 
; Teile aus Profimonitor rot entnommen (C)Rheinhard Goessler hms
; 
; (HL) Zeiger fuer Quellbuffer
;  (A) Hexwert
; 
; Fehler: (A)=00h + CY=1 kein ASCII zeichen)
;
asc_h:	push b
	push d
	mov a,m		; 1. zeichen holen
	call hxtst
	jc asc_err	; Fehler 
	rlc		; Nible in obere Haelfte
	rlc
	rlc
	rlc
	ani 0F0h	; unters Nible ausblenden
	mov b,a		;

	inx h		; 2. Zeichen holen
	mov a,m
	call hxtst
	jc asc_err	; Fehler 
	ani 0Fh		; oberes Nible ausblenden
	ora b		;
	
asc_err: pop d
	pop b
	ret		; (A) Hex
; 
hxtst: 	push b
	ani  5fh        ;Grossbuchstabe
       	cpi  10h        ;"0"
       	jc   hexerr     ;Code zu klein:JMP
       	cpi  1ah        ;"9"+1
       	jc   hexcte     ;Code o.k.:JMP
       	adi  0c9h       ;A...F adjust
       	cpi  10h        ;"F"+1
       	jnc  hexerr     ;Code zu gross:JMP
       	cpi  0ah        ;korr. "A"
       	jnc  insft      ;A...F: JMP
	
hexerr:	xra  a          ;FLAGs setzen
       	mvi  a,03h      ;kein HEX-Digit
	rar
	pop b
       	ret             ;Fehler: CY=1
;
hexcte:	sui  10h        ;Rest der "3x"
insft:	mov b,a
	xra a		;Flags löschen
	mov a,b
	pop b
 	ret             ;REG A: HEX-Byte
;
;--------------------------------------------------	
; Texte für Bildschirmmeldungen
;
meld_0:	.db 0dh		; CR  Startmeldung
	.db 0ah		; LF
	.db "Intel-Hex "
	.db "Ladeprogramm"
	.db " V2.51"
	.db 0dh		; CR
	.db "(C)"
	.db "W.Roemer 01.18"

meld_1:	.db 0dh		; CR
	.db 0ah		; LF
	.db "Sender starten"
	.db 0dh		; CR
	.db 0ah		; LF
	.db 00h		;  
	
meld_4: .db 0dh		; CR
	.db 0ah		; LF
	.db "Uebertragung"
	.db " beendet"
	.db 0dh		; CR
	.db 0ah		; LF
	.db 00h		; 
	
meld_5: .db 0dh		; CR
	.db 0ah		; LF
	.db "Hex-Interpreter"
	.db " fertig"
	.db 0dh		; CR
	.db 0ah		; LF
	.db 00h		; 
;----------------------------------------------------		
; Fehlermeldungen
;
err_0:	.db 0dh		;
	.db 0ah		;
	.db " Uebertragungs"
	.db "fehler"
	.db 0ah		; LF
	.db 00h		;
	
err_1: .db 0dh		; CR
	.db 0ah		; LF
	.db " kein ASCII"
	.db " Zeichen"
	.db 0dh		; CR
	.db 0ah		; LF
	.db 00h		;
	
err_2: .db 0dh		; CR
	.db 0ah		; LF
	.db " kein Startmarke"
	.db " gefunden"
	.db 0dh		; CR
	.db 0ah		; LF
	.db 00h		;

err_3: .db 0dh		; CR
	.db 0ah		; LF
	.db " Pruefsumme"
	.db " falsch"
	.db 0dh		; CR
	.db 0ah		; LF
	.db 00h		;	
	
err_5:	.db 0dh		;
	.db 0ah		;
	.db " Timeout"
	.db 0ah		; LF
	.db 00h		;
	
;--------------------------------------------------------------------------

	
	
	
	
