; Lader.asm V3.5
;
; Intel-Hex Datei ueber Serielle-Schnittstellenkarte lesen
;	Zeilenweise interpretieren (Zieladresse wird dem Hex-File entnommen)
;
; Stand:
; 26.06.2016 	V24 Anpassungen # V3.5 fuer CF-Test-V3.5
; 21.04.2016	Hex-Vorpann testen, nach 16Byte Fehlermeldung
; 19.04.2016    Umstellung auf Seriellesinterface der CF-Karte
;
; 14.03.2015	Mit "Bootlader" aus EPROM 4000h ins RAM ab 2900h kopieren  
; 12.03.2015	Leseroutine auf Zeilenbuffer umgestellt
; 17.02.2015	UART-Init ueberarbeitet und direkt am Programmanfang gesetzt
; 01.02.2015	auf 04000h assembliert - alter Platz vom Basicinterpreter
; 30.01.2015	Hex-Interpreter integriert
;		  keine Ueberpruefung der Buffergrenzen
; 27.01.2015	Zeitfenster eingebaut
;		  (Empfang wartet max 3sek auf Daten)
;		  (Senden wartet max 12ms auf Empfaenger)
;		  Buffergrenze auf 16kByte gesetzt
;		  Meldetexte
;
;-----------------------------------------------------------------------------
;
; PIO Ansteuerung
;
PIO_A	equ 	0cah			; CAh   Daten Kanal A
PIO_B	equ	0dah			; DAh   Daten Kanal B
PIO_C	equ	0eah			; EAh   Daten Kanal C
PIO_Ctl	equ	0fah			; FAh   Steuerwort
;
PIO_BA	equ	0c1h			; Betriebsart
					; Kanal A bidirektional, Handshake Kanal C Bit 3-7
;					; Kanal B als Ausgabe, Kanal C Bit 0-1
;
; AVR Befehle
;
AVRres	equ	00h			; Spur 0 Sektor 0 ansteuern
SetSpur equ	10h			; Spurnummer setzen
SetSekt equ	20h   			; Sektornummer setzen
Rdsekt	equ	30h			; Daten vom aktuellen Sektor lesen
Wrsekt	equ	40h			; Daten in den aktuellen Sektor schreiben

v24stat equ	5Fh			; Status der V24 Schnittstelle
					; Bit 0 CTS Signal
					; Bit 1 RTS Signal
					; Bit 5 Senderegister leer
					; Bit 7 Empfangsregister voll
v24run	equ	50h			; V24 vorbereiten (RTS setzen)
v24stop equ	51h			; 					
v24btx  equ	52h			; ein byte senden		ohne Buffe
v24brx	equ	53h			; ein byte empfangen
v24sttx	equ	54h			; String senden Ende 00h	mit Buffer
v24strx	equ	55h			; Zeile empfangen bis CR
;
AVRstat	equ	0F0h			; Status ermittel
;
;
;-----------------------------------------------------------------------------
;
;
buffer	EQU 2e10h	; Zeilenbuffer
SRCBEG	EQU 2FBFh	; Startadresse
SRCEND	EQU 2FC1h	; Endadresse
DSTBEG	EQU 2FC3h	; Anfang Zieladresse/Parameter
DSTEND	EQU 2FC5h	; 
RAMBEG	equ 37B0h	;
;
DIGI1	equ 2E00h	;
;
; Monitorprogramme
;
vwarm	equ 1003h	; Videomonitor
co	equ 0049h	; (C) Monitorausgabe
ci	equ 0043h	; (A) Tastencode, Rueksprung erst nach Betaetigung, wartet nicht auf loslassen
csts	equ 0052h	; Status Tastatur, gedruekt Z=0, sonst Z=1
dely1	equ 000bh	; warte 1ms
copy	equ 0088h	; SRCBEG=Quelle start,SRCEND=Quelle End,DSTBEG=Zielstart
onesec	equ 001bh	; warte 1 Sekunde
string	equ 1021h	; Textausgabe bis 00h
adrot	equ 103Ch	; (DE) vierstellig ausgeben (HL)=Position
error	EQU 1027h       ; Error Videomonitor
;
;==================================================================
;
	org 2900h	; Sprung vom Ladeprogramm 
	
lader:	jmp lade-	; Intel-Hex empfangen, interpretieren
hxi:	jmp hxi-	; Intel-Hex Interpreter fuer eine Zeile
bytrx:	jmp bytrx-	; ein Byte empfangen ueber CF-Karte
byttx:	jmp byttx-	; ein byte senden
strrx:	jmp strrx-	; Zeile bis CR empfangen
init:	jmp init-	; CF-Karte vorbereiten
inibu:	jmp inibu-	; Buffereinstellen

Ende:	call onesec	;
	jmp vwarm	; zurueck zum Monitor
;
;==================================================================
; Hauptprogramm
;
; - laden Intel-Hex-File Zeilenweise
; - Intel-Hex Interpreter
;

lade-:	call init-	; CF-Karte vorbereiten
			; Bildschirmmenue ausgeben
	lxi h,meld_0	; Meldung "Ladeprogramm" 
	call string
	lxi h,meld_1	; Start-Meldung Datenuebertragung 
	call string
	mvi  a,v24stop	; RTS setzen damit der Empfang nicht
	call piowr	; ueberrannt wird
	call onesec	; Wartezeit bis Datenabgeholt werden
	
lade_l:	call inibu-	;
	call strrx-	; Zeile empfangen
	cpi  00h	; Fehler auswerten
	jnz  lade_e4	; Bufferueberlauf
	;
	call inibu-	; Buffer auf Anfang
	;
	call hxi-	; eine Zeile interpretieren und im  Zielbuffer ablegen
	shld dstend	; Test
	;
	mvi c,2ah	; Sternchen pro Zeile ausgeben
	call co		;
	;
	cpi 01h		; Fehlercode auswerten
	jz lade_e1	; kein ASCII
	cpi 02h		;
	jz lade_e2	; keine Startmarke
	cpi 03h		;
	jz lade_e3	; falsche Pruefsumme
	cpi 04h		;
	jnz lade_l	; weiter bis Dateiende
	;
	lxi h, meld_5	; alles OK zurueck zum Monitor
lade_e: call string	; Bildschirmmeldung ausgeben
	mvi  a,v24run	; RTS freigeben
	call piowr	; 
	call onesec
	jmp Ende
	
lade_e0: lxi h,err_1	;
	jmp lade_e	;
	
lade_e1: lxi h,err_1	;
	jmp lade_e	;
	
lade_e2: lxi h,err_2	;
	jmp lade_e	;
	
lade_e3: lxi h,err_3	;
	jmp lade_e

lade_e4: lxi h,err_0	;
	jmp  lade_e
	
;==================================================================
;
; Unterprogramme
;
; Buffer initialisieren
; (HL) Buffer Startadresse
; (DE) Buffer Endadresse
;
inibu-:	lxi  h,buffer	; ZeilenBuffer max 64Byte
	ret

;-------------------------------------------------------------------------
;
; init = Reset Befehl an CF-Karte
;          
init-:	call	pioinit		; PIO und AVR initialisieren
	mvi	a,avrres	; Test ob CF-Erweiterung vorhanden ist;
	call	piowr		;
	xra	a		;
	call	piord		;
	cpi	00h		; 	
	jnz	cferr		; CF nicht bereit -> Fehler
	ret			; CF-Interface OK
;
; avr_stat = Status CF-Karte ermitteln 0=OK
;            sonst Fehlermeldung
;
avr_stat: mvi	a,avrstat	; Status AVR ermitteln
	call	piowr		; 
	call	piord		;
	cpi	00h		;
	jnz	cferr1		; Driver Error
	ret			;

;
cferr:	lxi	h,errmsg1	; Fehlermeldung 
	call	string		; CF nicht bereit
	mvi	a,01		;
	ret
		
cferr1:	lxi	h,errmsg	; Lese/schreibfehler
	mvi	a,27h		;
	call	error
	jmp	ende
;		
;-----------------------------------------------------------------------------
;
; PIO Initialisieren mit Betriebsart
; Kanal A bidirektional
; Kanal C Handshake
; Kanal B Ausgabe
;
pioinit:	push	psw		; PIO Betriebsart einstellen
		mvi	a,PIO_BA	; PIO_BA	equ	0c1h
		out	PIO_Ctl		; 
		pop	psw		;
		ret			;
;
;---------------------------------------------------------------------------
;
; PIO-A lesen (A)
; wartet bis Daten bereitstehen 

piord:	in	PIO_C		; Status ermitteln
	ani	00100000b	; IBF maskieren
	jz	piord		; warte auf Daten
	in	PIO_A		; daten lesen
	ret			; (A) Daten
;
; (A) PIO-A schreiben
; 

piowr:	push	psw		;
piosts:	in	PIO_C		; Status ermitteln
	ani	10000000b	; /OBF maskieren
	jz	piosts		; warten auf Quittung /ACK
	pop	psw
	out	PIO_A		; Daten schreiben
	ret			; 
;
;--------------------------------------------------------------------------
; Ein Byte lesen/schreiben
;
bytrx-:	mvi	a,v24brx	; Lesebefehl
	call	piowr
	call	piord		; Daten holen
	ret
	
byttx-:	push	psw		;
	mvi	a,v24btx	;
	call	piowr		; Schreibbefehl
	pop	psw		;
	call	piowr		;
	ret
;
; Zeile empfangen (bis CR max 64 Zeichen)
;

strrx-: lxi  h,buffer		; Buffer
	mvi  b,40h		; maxbuffer
	mvi a,v24strx		; 
	call piowr		;
	
str_l:	call piord		; Byte holen
	mov  m,a		;      speichern
	cpi  0dh		; ? CR
	jz   str_end		; Zeilenende
	inx  h
	dcr  b			;
	jnz  str_l		; weiter
str_er:	mvi  a,01h		; Fehler Bufferueberlauf
	ret			; Fehlercode 01h

str_end: xra a			; Zeile vollständig empfangen
	ret			; Fehlercode 00h
;		
;
;
;==============================================================================
; Eine Zeile Intel-Hex Daten aus RS232 Buffer lesen
;
; interpretieren und
; in Buffer legen
;
; eine Intel-Hex Zeile in 8Bit Systemen 
; ! jedes Byte ist in zwei ASCII-Zeichen codiert (z.B.10 = 31 30) !
;
;   I   Daten fuer Pruefsumme addieren inkl Pruefsumme = 0         I
;   I                                                           I
; : 10 2010 00 66 20 C3 88 23 C3 3B 24 D3 4B C9 1A D3 4B 13 C9 AF 0D 0A
; I I  I    I  I                                               I  I  I
; I I  I    I  1.Byte                                          I  I  Line-Feed
; I I  I    I                                                  I  I
; I I  I    Typ 00=Nutzdaten                                   I    CR
; I I  I        01=Ende der Inte-Hex datei (EOF)               I
; I I  I                                                       Pruefsumme
; I I  Startadresse
; I I
; I Anzahl Bytes
; I
; Startmarke
;
;
; Registerzuweisung:
;	
; (HL) Zeiger auf Lesebuffer
; (DE) Zeiger auf Schreibbuffer
; (C)  Bytezaehler
; (B)  Pruefsumme
;
;
;----------------------------------------------------
; Hex Interpreter wandelt eine Zeile Intel-HEX in Binaer-Code
; Anmerkung: Adressen werden pro Zeile ausgewertet, Zwischenraueme
;            werden nicht aufgefuellt - die Daten bleiben als Muell stehen 
; 
; Registerzuweisung:
;	
; (HL) Zeiger auf Lesebuffer
; (DE) Zeiger auf Schreibbuffer
; (C)  Bytezaehler
; (B)  Pruefsumme  
;
; Fehlercode:
;
; (A)=01h CY=1 kein ASII-Zeichen
; (A)=02h Z=1 Starmarke nicht gefunden
; (A)=02h Z=1 Pruefsumme falsch
; (A)=02h Z=1 Dateiende (EOF)
;
; Unterprogram: asc_h
;
; Startmarke ":" holen
;
hxi-:	xra a		; Registervorbereiten (DE+BC)
	mov b,a		;
	mov c,a		;
	mov d,a		;
	mov e,a		;
	push b		;
	mvi c,0fh	; Fehlerzaehler setzen
hxi_sl:	mov a,m		; Startmarke suchen
	cpi 3ah		;
	jz hxi_ln	; weiter mit Laenge holen
			; Zeilenende(abspann) behandeln
	cpi 0Dh		; test auf CR
	jz hxi_mt	;
	dcr c		; Fehlerzaehler testen
	jz hxi_erm 	; keine Startmarke gefunden
hxi_mt:	inx h		; nachstes Byte holen
	jmp hxi_sl	;

; Laenge holen

hxi_ln:	pop b		;
	inx h		; 2.Byte holen
	call asc_h	; ASCII->HEX
	jc hxi_era	; Fehler kein ASCII
	mov c,a		; Bytezaehler laden
	add b		;
	mov b,a		; Pruefsumme schreiben
	
; Adresse holen

	inx h		; HighByte
	call asc_h
	jc hxi_era	; Fehler kein ASCII
	mov d,a		;
	add b		;
	mov b,a		; Pruefsumme schreiben
	
	inx h		; LowByte
	call asc_h
	jc hxi_era	; Fehler kein ASCII
	mov e,a		; 
	add b		;
	mov b,a		; Pruefsumme schreiben

; Type holen

	inx h
	call asc_h
	jc hxi_era	; Fehler kein ASCII
	cpi 00h		; Nutzdaten folgen
	jnz hxi_erf	; EOF erreicht
	add b		;
	mov b,a		; Pruefsumme schreiben
	
; Daten holen

hxi_dl: inx h		;
	mov a,m		;
	call asc_h	;
	jc hxi_era	; Fehler kein ASCII
	push psw
	add b		;
	mov b,a		; Pruefsumme schreiben
	pop psw
	xchg		;
	mov m,a		; Datenbyte in Zielbuffer
	xchg		;
	inx d		;
	dcr c		;
	jnz hxi_dl	; naechstes Byte

; Pruefsumme holen

	inx h		;
	mov a,m		;
	call asc_h	;
	jc hxi_era	; Fehler kein ASCII
	add b		;
	cpi 0h		;
	jnz hxi_ers	; Fehler Pruefsumme
	inx h		;
hxi_end: ret 			
;
; Fehlerbehandlung
;
hxi_era: ret		; (A)=01h CY=1 kein ASII-Zeichen

hxi_erm: xra a		; (A)=02h Z=1 Starmarke nicht gefunden
	mvi a,02h
	ret		;
	
hxi_ers: xra a		; (A)=03h Z=1 Pruefsumme falsch
	mvi a,03h
	ret		;
	
hxi_erf: xra a		; (A)=04h Z=1Dateiende (EOF)
	mvi a,04h
	ret
;
;----------------------------------------------------
;
; ASCII -> HEX 
; Teile aus Profimonitor rot entnommen (C)Rheinhard Goessler hms
; 
; (HL) Zeiger fuer Quellbuffer
;  (A) Hexwert
; 
; Fehler: (A)=00h + CY=1 kein ASCII zeichen)
;
asc_h:	push b
	push d
	mov a,m		; 1. zeichen holen
	call hxtst
	jc asc_err	; Fehler 
	rlc		; Nible in obere Haelfte
	rlc
	rlc
	rlc
	ani 0F0h	; unters Nible ausblenden
	mov b,a		;

	inx h		; 2. Zeichen holen
	mov a,m
	call hxtst
	jc asc_err	; Fehler 
	ani 0Fh		; oberes Nible ausblenden
	ora b		;
	
asc_err: pop d
	pop b
	ret		; (A) Hex
; 
hxtst: 	push b
	ani  5fh        ;Grossbuchstabe
       	cpi  10h        ;"0"
       	jc   hexerr     ;Code zu klein:JMP
       	cpi  1ah        ;"9"+1
       	jc   hexcte     ;Code o.k.:JMP
       	adi  0c9h       ;A...F adjust
       	cpi  10h        ;"F"+1
       	jnc  hexerr     ;Code zu gross:JMP
       	cpi  0ah        ;korr. "A"
       	jnc  insft      ;A...F: JMP
	
hexerr:	xra  a          ;FLAGs setzen
       	mvi  a,03h      ;kein HEX-Digit
	rar
	pop b
       	ret             ;Fehler: CY=1
;
hexcte:	sui  10h        ;Rest der "3x"
insft:	mov b,a
	xra a		;Flags löschen
	mov a,b
	pop b
 	ret             ;REG A: HEX-Byte
;
;--------------------------------------------------	
; Texte für Bildschirmmeldungen
;
meld_0:	db 0dh		; CR  Startmeldung
	db 0ah		; LF
	db "Intel-Hex "
	db "Ladeprogramm"
	db " V3.5 (C)" 
	db "W.Roemer 06.16"
	db 00h		; Ende

meld_1:	db 0dh		; CR
	db 0ah		; LF
	db "Sender starten"
crlf:	db 0dh		; CR
	db 0ah		; LF
	db 00h		;  
	
meld_2:	db 0dh		; CR
	db 0ah		; LF
	db "Empfang Ende"
	db 0dh		; CR
	db 0ah		; LF
	db 00h		; 
	
meld_3:	db 0dh		; CR
	db 0ah		; LF
	db "Empfaenger"
	db " starten"	
	db 0dh		; CR
	db 0ah		; LF
	db 00h		; 
	
meld_4: db 0dh		; CR
	db 0ah		; LF
	db "Uebertragung"
	db " beendet"
	db 0dh		; CR
	db 0ah		; LF
	db 00h		; 
	
meld_5: db 0dh		; CR
	db 0ah		; LF
	db "Hex-Interpreter"
	db " fertig"
	db 0dh		; CR
	db 0ah		; LF
	db 00h		; 
;----------------------------------------------------		
; Fehlermeldungen
;
err_0:	db 0dh		;
	db 0ah		;
	db " Uebertragungs"
	db "fehler"
	db 0dh		; CR
	db 0ah		; LF
	db 00h		;
	
err_1: db 0dh		; CR
	db 0ah		; LF
	db " kein ASCII"
	db " Zeichen"
	db 0dh		; CR
	db 0ah		; LF
	db 00h		;
	
err_2: db 0dh		; CR
	db 0ah		; LF
	db " kein Startmarke"
	db " gefunden"
	db 0dh		; CR
	db 0ah		; LF
	db 00h		;

err_3: db 0dh		; CR
	db 0ah		; LF
	db " Pruefsumme"
	db " falsch"
	db 0dh		; CR
	db 0ah		; LF
	db 00h		;	
	
errmsg: DB 0Dh			; CR
	DB 0Ah			; LF     
	DB "Error F0"
	DB 0Dh			;
	DB 00h	 
	
errmsg1: DB 0Dh			; CR
	DB 0Ah			; LF     
	DB " CF-Karte "
	db "nicht vorhanden"
	DB 0Dh			;
	DB 00h	 
	
	
;--------------------------------------------------------------------------
;
; Ladeprogramm fuer Intel-Hex Lader
; Hex-Lader aus EPROM ab 4200h ins RAM 2900h
; 
;
	org	4000h 	;  

start:	jmp start-	; "Bootlader"
;
start-: lxi	h,4200h	; Quelle Startadr.
	shld	srcbeg	;
	lxi	h,44ffh	; Quelle Endadr.
	shld	srcend	;
	lxi	h,2900h	; Ziel Startadr.
	shld	dstbeg	;
	call	copy	;
	
	jmp	2900h	;


	
	
	
	
